@@ -21,25 +21,47 @@
 """Functionality related to packages."""
 from __future__ import print_function
 
-
+import logging
 import os
 import sys
 import re
 import socket
 import subprocess
+import threading
+threading  # pyflakes
 
 try:
     from http.client import BadStatusLine
     from urllib.error import HTTPError
     from urllib.request import urlopen
 except ImportError:
-    from httplib import BadStatusLine
-    from urllib2 import HTTPError, urlopen
+    from httplib import BadStatusLine  # type: ignore
+    from urllib2 import HTTPError, urlopen  # type: ignore
 
 from collections import Mapping, Sequence
 
+try:
+    from typing import Any, Iterable, Iterator, List, Set, Tuple, Union
+    Any  # pyflakes
+    Iterable  # pyflakes
+    Iterator  # pyflakes
+    List  # pyflakes
+    Set  # pyflakes
+    Tuple  # pyflakes
+    Union  # pyflakes
+except ImportError:
+    pass
+
 import apt_pkg
 import apt.progress.text
+
+from apt.progress.base import (
+    AcquireProgress,
+    InstallProgress,
+)
+AcquireProgress  # pyflakes
+InstallProgress  # pyflakes
+
 from apt_pkg import gettext as _
 
 __all__ = ('BaseDependency', 'Dependency', 'Origin', 'Package', 'Record',
@@ -49,11 +71,12 @@ if sys.version_info.major >= 3:
     unicode = str
 
 
-def _file_is_same(path, size, hashes):
+def _file_is_same(path, size, md5):
+    # type: (str, int, str) -> bool
     """Return ``True`` if the file is the same."""
     if os.path.exists(path) and os.path.getsize(path) == size:
         with open(path) as fobj:
-            return apt_pkg.Hashes(fobj).hashes == hashes
+            return apt_pkg.md5sum(fobj) == md5
     return False
 
 
@@ -61,10 +84,6 @@ class FetchError(Exception):
     """Raised when a file could not be fetched."""
 
 
-class UntrustedError(FetchError):
-    """Raised when a file did not have a trusted hash."""
-
-
 class BaseDependency(object):
     """A single dependency."""
 
@@ -93,23 +112,28 @@ class BaseDependency(object):
             return not self.__eq__(other)
 
     def __init__(self, version, dep):
+        # type: (Version, apt_pkg.Dependency) -> None
         self._version = version  # apt.package.Version
         self._dep = dep  # apt_pkg.Dependency
 
     def __str__(self):
+        # type: () -> str
         return '%s: %s' % (self.rawtype, self.rawstr)
 
     def __repr__(self):
+        # type: () -> str
         return ('<BaseDependency: name:%r relation:%r version:%r rawtype:%r>'
                 % (self.name, self.relation, self.version, self.rawtype))
 
     @property
     def name(self):
+        # type: () -> str
         """The name of the target package."""
         return self._dep.target_pkg.name
 
     @property
     def relation(self):
+        # type: () -> str
         """The relation (<, <=, =, !=, >=, >, '') in mathematical notation.
 
         The empty string will be returned in case of an unversioned dependency.
@@ -118,6 +142,7 @@ class BaseDependency(object):
 
     @property
     def relation_deb(self):
+        # type: () -> str
         """The relation (<<, <=, =, !=, >=, >>, '') in Debian notation.
 
         The empty string will be returned in case of an unversioned dependency.
@@ -131,6 +156,7 @@ class BaseDependency(object):
 
     @property
     def version(self):
+        # type: () -> str
         """The target version or an empty string.
 
         Note that the version is only an empty string in case of an unversioned
@@ -140,6 +166,7 @@ class BaseDependency(object):
 
     @property
     def target_versions(self):
+        # type: () -> List[Version]
         """A list of all Version objects which satisfy this dependency.
 
         .. versionadded:: 1.0.0
@@ -156,6 +183,7 @@ class BaseDependency(object):
 
     @property
     def installed_target_versions(self):
+        # type: () -> List[Version]
         """A list of all installed Version objects which satisfy this dep.
 
         .. versionadded:: 1.0.0
@@ -164,6 +192,7 @@ class BaseDependency(object):
 
     @property
     def rawstr(self):
+        # type: () -> str
         """String represenation of the dependency.
 
         Returns the string representation of the dependency as it would be
@@ -185,6 +214,7 @@ class BaseDependency(object):
 
     @property
     def rawtype(self):
+        # type: () -> str
         """Type of the dependency.
 
         This should be one of 'Breaks', 'Conflicts', 'Depends', 'Enhances',
@@ -196,6 +226,7 @@ class BaseDependency(object):
 
     @property
     def pre_depend(self):
+        # type: () -> bool
         """Whether this is a PreDepends."""
         return self._dep.dep_type_untranslated == 'PreDepends'
 
@@ -211,22 +242,27 @@ class Dependency(list):
     """
 
     def __init__(self, version, base_deps, rawtype):
-        super(Dependency, self).__init__(base_deps)
+        # type: (Version, apt_pkg.Dependency, str) -> None
+        super(Dependency, self).__init__(base_deps)  # type: ignore
         self._version = version  # apt.package.Version
         self._rawtype = rawtype
 
     def __str__(self):
+        # type: () -> str
         return '%s: %s' % (self.rawtype, self.rawstr)
 
     def __repr__(self):
+        # type: () -> str
         return '<Dependency: [%s]>' % (', '.join(repr(bd) for bd in self))
 
     @property
     def or_dependencies(self):
+        # type: () -> Dependency
         return self
 
     @property
     def rawstr(self):
+        # type: () -> str
         """String represenation of the Or-group of dependencies.
 
         Returns the string representation of the Or-group of dependencies as it
@@ -242,6 +278,7 @@ class Dependency(list):
 
     @property
     def rawtype(self):
+        # type: () -> str
         """Type of the Or-group of dependency.
 
         This should be one of 'Breaks', 'Conflicts', 'Depends', 'Enhances',
@@ -255,11 +292,12 @@ class Dependency(list):
 
     @property
     def target_versions(self):
+        # type: () -> List[Version]
         """A list of all Version objects which satisfy this Or-group of deps.
 
         .. versionadded:: 1.0.0
         """
-        tvers = []
+        tvers = []  # type: List[Version]
         for bd in self:  # apt.package.Dependency
             for tver in bd.target_versions:  # apt.package.Version
                 if tver not in tvers:
@@ -268,6 +306,7 @@ class Dependency(list):
 
     @property
     def installed_target_versions(self):
+        # type: () -> List[Version]
         """A list of all installed Version objects which satisfy this dep.
 
         .. versionadded:: 1.0.0
@@ -289,6 +328,7 @@ class Origin(object):
     """
 
     def __init__(self, pkg, packagefile):
+        # type: (Package, apt_pkg.PackageFile) -> None
         self.archive = packagefile.archive
         self.component = packagefile.component
         self.label = packagefile.label
@@ -304,6 +344,7 @@ class Origin(object):
             self.trusted = False
 
     def __repr__(self):
+        # type: () -> str
         return ("<Origin component:%r archive:%r origin:%r label:%r "
                 "site:%r isTrusted:%r>") % (self.component, self.archive,
                                             self.origin, self.label,
@@ -331,29 +372,37 @@ class Record(Mapping):
     """
 
     def __init__(self, record_str):
+        # type: (str) -> None
         self._rec = apt_pkg.TagSection(record_str)
 
     def __hash__(self):
+        # type: () -> Any
         return hash(self._rec)
 
     def __str__(self):
+        # type: () -> str
         return str(self._rec)
 
     def __getitem__(self, key):
+        # type: (str) -> str
         return self._rec[key]
 
     def __contains__(self, key):
+        # type: (object) -> bool
         return key in self._rec
 
     def __iter__(self):
+        # type: () -> Iterator[str]
         return iter(self._rec.keys())
 
     def iteritems(self):
+        # type: () -> Iterable
         """An iterator over the (key, value) items of the record."""
         for key in self._rec.keys():
             yield key, self._rec[key]
 
     def get(self, key, default=None):
+        # type: (object, object) -> object
         """Return record[key] if key in record, else *default*.
 
         The parameter *default* must be either a string or None.
@@ -361,10 +410,12 @@ class Record(Mapping):
         return self._rec.get(key, default)
 
     def has_key(self, key):
+        # type: (str) -> bool
         """deprecated form of ``key in x``."""
         return key in self._rec
 
     def __len__(self):
+        # type: () -> int
         return len(self._rec)
 
 
@@ -378,10 +429,12 @@ class Version(object):
     """
 
     def __init__(self, package, cand):
+        # type: (Package, apt_pkg.Version) -> None
         self.package = package
         self._cand = cand
 
     def _cmp(self, other):
+        # FIXME: add type hint
         """Compares against another apt.Version object or a version string.
 
         This method behaves like Python 2's cmp builtin and returns an integer
@@ -411,45 +464,56 @@ class Version(object):
                 return NotImplemented
 
     def __eq__(self, other):
+        # type: (object) -> bool
         return self._cmp(other) == 0
 
     def __ge__(self, other):
+        # type: (Version) -> bool
         return self._cmp(other) >= 0
 
     def __gt__(self, other):
+        # type: (Version) -> bool
         return self._cmp(other) > 0
 
     def __le__(self, other):
+        # type: (Version) -> bool
         return self._cmp(other) <= 0
 
     def __lt__(self, other):
+        # type: (Version) -> bool
         return self._cmp(other) < 0
 
     def __ne__(self, other):
+        # type: (object) -> bool
         try:
             return self._cmp(other) != 0
         except TypeError:
             return NotImplemented
 
     def __hash__(self):
+        # type: () -> Any
         return self._cand.hash
 
     def __str__(self):
+        # type: () -> str
         return '%s=%s' % (self.package.name, self.version)
 
     def __repr__(self):
+        # type: () -> str
         return '<Version: package:%r version:%r>' % (self.package.name,
                                                      self.version)
 
     @property
     def _records(self):
+        # type: () -> apt_pkg.PackageRecords
         """Internal helper that moves the Records to the right position."""
-        # If changing lookup, change fetch_binary() as well
         if self.package._pcache._records.lookup(self._cand.file_list[0]):
             return self.package._pcache._records
+        return None
 
     @property
     def _translated_records(self):
+        # type: () -> apt_pkg.PackageRecords
         """Internal helper to get the translated description."""
         desc_iter = self._cand.translated_description
         self.package._pcache._records.lookup(desc_iter.file_list.pop(0))
@@ -457,60 +521,71 @@ class Version(object):
 
     @property
     def installed_size(self):
+        # type: () -> int
         """Return the size of the package when installed."""
         return self._cand.installed_size
 
     @property
     def homepage(self):
+        # type: () -> str
         """Return the homepage for the package."""
         return self._records.homepage
 
     @property
     def size(self):
+        # type: () -> int
         """Return the size of the package."""
         return self._cand.size
 
     @property
     def architecture(self):
+        # type: () -> str
         """Return the architecture of the package version."""
         return self._cand.arch
 
     @property
     def downloadable(self):
+        # type: () -> bool
         """Return whether the version of the package is downloadable."""
         return bool(self._cand.downloadable)
 
     @property
     def is_installed(self):
+        # type: () -> bool
         """Return wether this version of the package is currently installed.
 
         .. versionadded:: 1.0.0
         """
         inst_ver = self.package.installed
-        return inst_ver and inst_ver._cand.id == self._cand.id
+        return (inst_ver is not None and inst_ver._cand.id == self._cand.id)
 
     @property
     def version(self):
+        # type: () -> str
         """Return the version as a string."""
         return self._cand.ver_str
 
     @property
     def summary(self):
+        # type: () -> str
         """Return the short description (one line summary)."""
         return self._translated_records.short_desc
 
     @property
     def raw_description(self):
+        # type: () -> str
         """return the long description (raw)."""
         return self._records.long_desc
 
     @property
     def section(self):
+        # type: () -> str
         """Return the section of the package."""
         return self._cand.section
 
     @property
     def description(self):
+        # type: () -> str
         """Return the formatted long description.
 
         Return the formatted long description according to the Debian policy
@@ -558,6 +633,7 @@ class Version(object):
 
     @property
     def source_name(self):
+        # type: () -> str
         """Return the name of the source package."""
         try:
             return self._records.source_pkg or self.package.shortname
@@ -566,6 +642,7 @@ class Version(object):
 
     @property
     def source_version(self):
+        # type: () -> str
         """Return the version of the source package."""
         try:
             return self._records.source_ver or self._cand.ver_str
@@ -574,11 +651,13 @@ class Version(object):
 
     @property
     def priority(self):
+        # type: () -> str
         """Return the priority of the package, as string."""
         return self._cand.priority_str
 
     @property
     def policy_priority(self):
+        # type: () -> int
         """Return the internal policy priority as a number.
            See apt_preferences(5) for more information about what it means.
         """
@@ -590,6 +669,7 @@ class Version(object):
 
     @property
     def record(self):
+        # type: () -> Record
         """Return a Record() object for this version.
 
         Return a Record() object for this version which provides access
@@ -598,6 +678,7 @@ class Version(object):
         return Record(self._records.record)
 
     def get_dependencies(self, *types):
+        # FIXME: add type hints
         """Return a list of Dependency objects for the given types.
 
         Multiple types can be specified. Possible types are:
@@ -621,31 +702,37 @@ class Version(object):
 
     @property
     def provides(self):
+        # type: () -> List[str]
         """ Return a list of names that this version provides."""
         return [p[0] for p in self._cand.provides_list]
 
     @property
     def enhances(self):
+        # type: () -> List[Dependency]
         """Return the list of enhances for the package version."""
         return self.get_dependencies("Enhances")
 
     @property
     def dependencies(self):
+        # type: () -> List[Dependency]
         """Return the dependencies of the package version."""
         return self.get_dependencies("PreDepends", "Depends")
 
     @property
     def recommends(self):
+        # type: () -> List[Dependency]
         """Return the recommends of the package version."""
         return self.get_dependencies("Recommends")
 
     @property
     def suggests(self):
+        # type: () -> List[Dependency]
         """Return the suggests of the package version."""
         return self.get_dependencies("Suggests")
 
     @property
     def origins(self):
+        # type: () -> List[Origin]
         """Return a list of origins for the package version."""
         origins = []
         for (packagefile, _unused) in self._cand.file_list:
@@ -654,6 +741,7 @@ class Version(object):
 
     @property
     def filename(self):
+        # type: () -> str
         """Return the path to the file inside the archive.
 
         .. versionadded:: 0.7.10
@@ -662,6 +750,7 @@ class Version(object):
 
     @property
     def md5(self):
+        # type: () -> str
         """Return the md5sum of the binary.
 
         .. versionadded:: 0.7.10
@@ -670,6 +759,7 @@ class Version(object):
 
     @property
     def sha1(self):
+        # type: () -> str
         """Return the sha1sum of the binary.
 
         .. versionadded:: 0.7.10
@@ -678,6 +768,7 @@ class Version(object):
 
     @property
     def sha256(self):
+        # type: () -> str
         """Return the sha256sum of the binary.
 
         .. versionadded:: 0.7.10
@@ -686,6 +777,7 @@ class Version(object):
 
     @property
     def tasks(self):
+        # type: () -> Set[str]
         """Get the tasks of the package.
 
         A set of the names of the tasks this package belongs to.
@@ -695,6 +787,7 @@ class Version(object):
         return set(self.record["Task"].split())
 
     def _uris(self):
+        # type: () -> Iterator[str]
         """Return an iterator over all available urls.
 
         .. versionadded:: 0.7.10
@@ -706,6 +799,7 @@ class Version(object):
 
     @property
     def uris(self):
+        # type: () -> List[str]
         """Return a list of all available uris for the binary.
 
         .. versionadded:: 0.7.10
@@ -714,6 +808,7 @@ class Version(object):
 
     @property
     def uri(self):
+        # type: () -> str
         """Return a single URI for the binary.
 
         .. versionadded:: 0.7.10
@@ -723,8 +818,8 @@ class Version(object):
         except StopIteration:
             return None
 
-    def fetch_binary(self, destdir='', progress=None,
-                     allow_unauthenticated=None):
+    def fetch_binary(self, destdir='', progress=None):
+        # type: (str, AcquireProgress) -> str
         """Fetch the binary version of the package.
 
         The parameter *destdir* specifies the directory where the package will
@@ -734,39 +829,15 @@ class Version(object):
         object. If not specified or None, apt.progress.text.AcquireProgress()
         is used.
 
-        The keyword-only parameter *allow_unauthenticated* specifies whether
-        to allow unauthenticated downloads. If not specified, it defaults to
-        the configuration option `APT::Get::AllowUnauthenticated`.
-
         .. versionadded:: 0.7.10
         """
-        if allow_unauthenticated is None:
-            allow_unauthenticated = apt_pkg.config.find_b("APT::Get::"
-                                        "AllowUnauthenticated", False)
         base = os.path.basename(self._records.filename)
         destfile = os.path.join(destdir, base)
-        if _file_is_same(destfile, self.size, self._records.hashes):
-            print(('Ignoring already existing file: %s' % destfile))
+        if _file_is_same(destfile, self.size, self._records.md5_hash):
+            logging.debug('Ignoring already existing file: %s' % destfile)
             return os.path.abspath(destfile)
-
-        # Verify that the index is actually trusted
-        pfile, offset = self._cand.file_list[0]
-        index = self.package._pcache._list.find_index(pfile)
-
-        if not (allow_unauthenticated or (index and index.is_trusted)):
-            raise UntrustedError("Could not fetch %s %s source package: "
-                                 "Source %r is not trusted" %
-                                 (self.package.name, self.version,
-                                  getattr(index, "describe", "<unkown>")))
-        if not self.uri:
-            raise ValueError("No URI for this binary.")
-        hashes = self._records.hashes
-        if not (allow_unauthenticated or hashes.usable):
-            raise UntrustedError("The item %r could not be fetched: "
-                                     "No trusted hash found." %
-                                     destfile)
         acq = apt_pkg.Acquire(progress or apt.progress.text.AcquireProgress())
-        acqfile = apt_pkg.AcquireFile(acq, self.uri, hashes,
+        acqfile = apt_pkg.AcquireFile(acq, self.uri, self._records.md5_hash,
                                       self.size, base, destfile=destfile)
         acq.run()
 
@@ -776,8 +847,8 @@ class Version(object):
 
         return os.path.abspath(destfile)
 
-    def fetch_source(self, destdir="", progress=None, unpack=True,
-                     allow_unauthenticated=None):
+    def fetch_source(self, destdir="", progress=None, unpack=True):
+        # type: (str, AcquireProgress, bool) -> str
         """Get the source code of a package.
 
         The parameter *destdir* specifies the directory where the source will
@@ -792,15 +863,7 @@ class Version(object):
 
         If *unpack* is ``True``, the path to the extracted directory is
         returned. Otherwise, the path to the .dsc file is returned.
-
-        The keyword-only parameter *allow_unauthenticated* specifies whether
-        to allow unauthenticated downloads. If not specified, it defaults to
-        the configuration option `APT::Get::AllowUnauthenticated`.
         """
-        if allow_unauthenticated is None:
-            allow_unauthenticated = apt_pkg.config.find_b("APT::Get::"
-                                        "AllowUnauthenticated", False)
-
         src = apt_pkg.SourceRecords()
         acq = apt_pkg.Acquire(progress or apt.progress.text.AcquireProgress())
 
@@ -815,28 +878,16 @@ class Version(object):
         if not source_lookup:
             raise ValueError("No source for %r" % self)
         files = list()
-
-        if not (allow_unauthenticated or src.index.is_trusted):
-            raise UntrustedError("Could not fetch %s %s source package: "
-                                 "Source %r is not trusted" %
-                                 (self.package.name, self.version,
-                                  src.index.describe))
-        for fil in src.files:
-            base = os.path.basename(fil.path)
+        for md5, size, path, type_ in src.files:
+            base = os.path.basename(path)
             destfile = os.path.join(destdir, base)
-            if fil.type == 'dsc':
+            if type_ == 'dsc':
                 dsc = destfile
-            if _file_is_same(destfile, fil.size, fil.hashes):
-                print(('Ignoring already existing file: %s' % destfile))
+            if _file_is_same(destfile, size, md5):
+                logging.debug('Ignoring already existing file: %s' % destfile)
                 continue
-
-            if not (allow_unauthenticated or fil.hashes.usable):
-                raise UntrustedError("The item %r could not be fetched: "
-                                         "No trusted hash found." %
-                                         destfile)
-            files.append(apt_pkg.AcquireFile(acq,
-                            src.index.archive_uri(fil.path),
-                            fil.hashes, fil.size, base, destfile=destfile))
+            files.append(apt_pkg.AcquireFile(acq, src.index.archive_uri(path),
+                         md5, size, base, destfile=destfile))
         acq.run()
 
         for item in acq.items:
@@ -874,12 +925,14 @@ class VersionList(Sequence):
     """
 
     def __init__(self, package, slice_=None):
+        # type: (Package, Any) -> None
         self._package = package  # apt.package.Package()
         self._versions = package._pkg.version_list  # [apt_pkg.Version(), ...]
         if slice_:
             self._versions = self._versions[slice_]
 
     def __getitem__(self, item):
+        # FIXME: add type hints
         if isinstance(item, slice):
             return self.__class__(self._package, item)
         try:
@@ -893,16 +946,20 @@ class VersionList(Sequence):
         raise KeyError("Version: %r not found." % (item))
 
     def __str__(self):
+        # type: () -> str
         return '[%s]' % (', '.join(str(ver) for ver in self))
 
     def __repr__(self):
+        # type: () -> str
         return '<VersionList: %r>' % self.keys()
 
     def __iter__(self):
+        # type: () -> Iterator[Version]
         """Return an iterator over all value objects."""
         return (Version(self._package, ver) for ver in self._versions)
 
     def __contains__(self, item):
+        # type: (object) -> bool
         if isinstance(item, Version):  # Sequence interface
             item = item.version
         # Dictionary interface.
@@ -912,18 +969,22 @@ class VersionList(Sequence):
         return False
 
     def __eq__(self, other):
+        # type: (Any) -> bool
         return list(self) == list(other)
 
     def __len__(self):
+        # type: () -> int
         return len(self._versions)
 
     # Mapping interface
 
     def keys(self):
+        # type: () -> List[str]
         """Return a list of all versions, as strings."""
         return [ver.ver_str for ver in self._versions]
 
     def get(self, key, default=None):
+        # type: (str, Version) -> Version
         """Return the key or the default."""
         try:
             return self[key]
@@ -940,22 +1001,27 @@ class Package(object):
     """
 
     def __init__(self, pcache, pkgiter):
+        # type: (apt.Cache, apt_pkg.Package) -> None
         """ Init the Package object """
         self._pkg = pkgiter
         self._pcache = pcache           # python cache in cache.py
         self._changelog = ""            # Cached changelog
 
     def __str__(self):
+        # type: () -> str
         return self.name
 
     def __repr__(self):
+        # type: () -> str
         return '<Package: name:%r architecture=%r id:%r>' % (
             self._pkg.name, self._pkg.architecture, self._pkg.id)
 
     def __lt__(self, other):
+        # type: (Package) -> bool
         return self.name < other.name
 
-    def candidate(self):
+    def __get_candidate(self):
+        # type: () -> Version
         """Return the candidate version of the package.
 
         This property is writeable to allow you to set the candidate version
@@ -965,26 +1031,31 @@ class Package(object):
         cand = self._pcache._depcache.get_candidate_ver(self._pkg)
         if cand is not None:
             return Version(self, cand)
+        return None
 
     def __set_candidate(self, version):
+        # type: (Version) -> None
         """Set the candidate version of the package."""
         self._pcache.cache_pre_change()
         self._pcache._depcache.set_candidate_ver(self._pkg, version._cand)
         self._pcache.cache_post_change()
 
-    candidate = property(candidate, __set_candidate)
+    candidate = property(__get_candidate, __set_candidate)
 
     @property
     def installed(self):
+        # type: () -> Version
         """Return the currently installed version of the package.
 
         .. versionadded:: 0.7.9
         """
         if self._pkg.current_ver is not None:
             return Version(self, self._pkg.current_ver)
+        return None
 
     @property
     def name(self):
+        # type: () -> str
         """Return the name of the package, possibly including architecture.
 
         If the package is not part of the system's preferred architecture,
@@ -1000,13 +1071,18 @@ class Package(object):
 
     @property
     def fullname(self):
+        # type: () -> str
         """Return the name of the package, including architecture.
 
+        Note that as for :meth:`architecture`, this returns the
+        native architecture for Architecture: all packages.
+
         .. versionadded:: 0.7.100.3"""
         return self._pkg.get_fullname(False)
 
     @property
     def shortname(self):
+        # type: () -> str
         """Return the name of the package, without architecture.
 
         .. versionadded:: 0.7.100.3"""
@@ -1014,12 +1090,14 @@ class Package(object):
 
     @property
     def id(self):
+        # type: () -> int
         """Return a uniq ID for the package.
 
         This can be used eg. to store additional information about the pkg."""
         return self._pkg.id
 
     def __hash__(self):
+        # type: () -> int
         """Return the hash of the object.
 
         This returns the same value as ID, which is unique."""
@@ -1027,12 +1105,19 @@ class Package(object):
 
     @property
     def essential(self):
+        # type: () -> bool
         """Return True if the package is an essential part of the system."""
         return self._pkg.essential
 
     def architecture(self):
+        # type: () -> str
         """Return the Architecture of the package.
 
+        Note that for Architecture: all packages, this returns the
+        native architecture, as they are internally treated like native
+        packages. To get the concrete architecture, look at the
+        :attr:`Version.architecture` attribute.
+
         .. versionchanged:: 0.7.100.3
             This is now the package's architecture in the multi-arch sense,
             previously it was the architecture of the candidate version
@@ -1042,6 +1127,7 @@ class Package(object):
 
     @property
     def section(self):
+        # type: () -> str
         """Return the section of the package."""
         return self._pkg.section
 
@@ -1049,47 +1135,56 @@ class Package(object):
 
     @property
     def marked_install(self):
+        # type: () -> bool
         """Return ``True`` if the package is marked for install."""
         return self._pcache._depcache.marked_install(self._pkg)
 
     @property
     def marked_upgrade(self):
+        # type: () -> bool
         """Return ``True`` if the package is marked for upgrade."""
         return self._pcache._depcache.marked_upgrade(self._pkg)
 
     @property
     def marked_delete(self):
+        # type: () -> bool
         """Return ``True`` if the package is marked for delete."""
         return self._pcache._depcache.marked_delete(self._pkg)
 
     @property
     def marked_keep(self):
+        # type: () -> bool
         """Return ``True`` if the package is marked for keep."""
         return self._pcache._depcache.marked_keep(self._pkg)
 
     @property
     def marked_downgrade(self):
+        # type: () -> bool
         """ Package is marked for downgrade """
         return self._pcache._depcache.marked_downgrade(self._pkg)
 
     @property
     def marked_reinstall(self):
+        # type: () -> bool
         """Return ``True`` if the package is marked for reinstall."""
         return self._pcache._depcache.marked_reinstall(self._pkg)
 
     @property
     def is_installed(self):
+        # type: () -> bool
         """Return ``True`` if the package is installed."""
         return (self._pkg.current_ver is not None)
 
     @property
     def is_upgradable(self):
+        # type: () -> bool
         """Return ``True`` if the package is upgradable."""
         return (self.is_installed and
                 self._pcache._depcache.is_upgradable(self._pkg))
 
     @property
     def is_auto_removable(self):
+        # type: () -> bool
         """Return ``True`` if the package is no longer required.
 
         If the package has been installed automatically as a dependency of
@@ -1101,12 +1196,14 @@ class Package(object):
 
     @property
     def is_auto_installed(self):
+        # type: () -> bool
         """Return whether the package is marked as automatically installed."""
         return self._pcache._depcache.is_auto_installed(self._pkg)
     # sizes
 
     @property
     def installed_files(self):
+        # type: () -> List[str]
         """Return a list of files installed by the package.
 
         Return a list of unicode names of the files which have
@@ -1123,6 +1220,7 @@ class Package(object):
         return []
 
     def get_changelog(self, uri=None, cancel_lock=None):
+        # type: (str, threading.Event) -> str
         """
         Download the changelog of the package and return it as unicode
         string.
@@ -1136,7 +1234,7 @@ class Package(object):
                 "/%(src_section)s/%(prefix)s/%(src_pkg)s" \\
                 "/%(src_pkg)s_%(src_ver)s/changelog"
 
-        The parameter *cancel_lock* refers to an instance of threading.Lock,
+        The parameter *cancel_lock* refers to an instance of threading.Event,
         which if set, prevents the download.
         """
         # Return a cached changelog if available
@@ -1156,7 +1254,10 @@ class Package(object):
                       "/%(src_pkg)s_%(src_ver)s/changelog"
             else:
                 res = _("The list of changes is not available")
-                return res if isinstance(res, unicode) else res.decode("utf-8")
+                if isinstance(res, unicode):
+                    return res
+                else:
+                    return res.decode("utf-8")
 
         # get the src package name
         src_pkg = self.candidate.source_name
@@ -1222,7 +1323,7 @@ class Package(object):
                 socket.setdefaulttimeout(2)
 
                 # Check if the download was canceled
-                if cancel_lock and cancel_lock.isSet():
+                if cancel_lock and cancel_lock.is_set():
                     return u""
                 # FIXME: python3.2: Should be closed manually
                 changelog_file = urlopen(uri)
@@ -1231,7 +1332,7 @@ class Package(object):
                 regexp = "^%s \((.*)\)(.*)$" % (re.escape(src_pkg))
                 while True:
                     # Check if the download was canceled
-                    if cancel_lock and cancel_lock.isSet():
+                    if cancel_lock and cancel_lock.is_set():
                         return u""
                     # Read changelog line by line
                     line_raw = changelog_file.readline()
@@ -1276,17 +1377,24 @@ class Package(object):
                             "later.") % (src_pkg, src_ver)
                 else:
                     res = _("The list of changes is not available")
-                return res if isinstance(res, unicode) else res.decode("utf-8")
+                if isinstance(res, unicode):
+                    return res
+                else:
+                    return res.decode("utf-8")
             except (IOError, BadStatusLine):
                 res = _("Failed to download the list of changes. \nPlease "
                         "check your Internet connection.")
-                return res if isinstance(res, unicode) else res.decode("utf-8")
+                if isinstance(res, unicode):
+                    return res
+                else:
+                    return res.decode("utf-8")
         finally:
             socket.setdefaulttimeout(timeout)
         return self._changelog
 
     @property
     def versions(self):
+        # type: () -> VersionList
         """Return a VersionList() object for all available versions.
 
         .. versionadded:: 0.7.9
@@ -1295,28 +1403,33 @@ class Package(object):
 
     @property
     def is_inst_broken(self):
+        # type: () -> bool
         """Return True if the to-be-installed package is broken."""
         return self._pcache._depcache.is_inst_broken(self._pkg)
 
     @property
     def is_now_broken(self):
+        # type: () -> bool
         """Return True if the installed package is broken."""
         return self._pcache._depcache.is_now_broken(self._pkg)
 
     @property
     def has_config_files(self):
+        # type: () -> bool
         """Checks whether the package is is the config-files state."""
         return self. _pkg.current_state == apt_pkg.CURSTATE_CONFIG_FILES
 
     # depcache actions
 
     def mark_keep(self):
+        # type: () -> None
         """Mark a package for keep."""
         self._pcache.cache_pre_change()
         self._pcache._depcache.mark_keep(self._pkg)
         self._pcache.cache_post_change()
 
     def mark_delete(self, auto_fix=True, purge=False):
+        # type: (bool, bool) -> None
         """Mark a package for deletion.
 
         If *auto_fix* is ``True``, the resolver will be run, trying to fix
@@ -1338,6 +1451,7 @@ class Package(object):
         self._pcache.cache_post_change()
 
     def mark_install(self, auto_fix=True, auto_inst=True, from_user=True):
+        # type: (bool, bool, bool) -> None
         """Mark a package for install.
 
         If *autoFix* is ``True``, the resolver will be run, trying to fix
@@ -1362,6 +1476,7 @@ class Package(object):
         self._pcache.cache_post_change()
 
     def mark_upgrade(self, from_user=True):
+        # type: (bool) -> None
         """Mark a package for upgrade."""
         if self.is_upgradable:
             auto = self.is_auto_installed
@@ -1373,6 +1488,7 @@ class Package(object):
                               "'%s'\n") % self._pkg.name)
 
     def mark_auto(self, auto=True):
+        # type: (bool) -> None
         """Mark a package as automatically installed.
 
         Call this function to mark a package as automatically installed. If the
@@ -1382,6 +1498,7 @@ class Package(object):
         self._pcache._depcache.mark_auto(self._pkg, auto)
 
     def commit(self, fprogress, iprogress):
+        # type: (AcquireProgress, InstallProgress) -> None
         """Commit the changes.
 
         The parameter *fprogress* refers to a apt_pkg.AcquireProgress() object,
@@ -1449,6 +1566,7 @@ def _test():
         print("Broken: %s " % cache._depcache.broken_count)
         print("DelCount: %s " % cache._depcache.del_count)
 
+
 # self-test
 if __name__ == "__main__":
     _test()
