@@ -34,6 +34,13 @@
 #endif
 
 
+#define VALIDATE_ITERATOR(I) do { \
+   if ((I).Cache() != &depcache->GetCache()) { \
+      PyErr_SetString(PyExc_ValueError, "Object of different cache passed as argument to apt_pkg.DepCache method"); \
+      return nullptr; \
+   } \
+} while(0)
+
 
 // DepCache Class								/*{{{*/
 // ---------------------------------------------------------------------
@@ -206,6 +213,7 @@ static PyObject *PkgDepCacheSetCandidateRelease(PyObject *Self,PyObject *Args)
    if(I.end()) {
       return HandleErrors(PyBool_FromLong(false));
    }
+   VALIDATE_ITERATOR(I);
 
    Success = depcache->SetCandidateRelease(I, target_rel, Changed);
 
@@ -222,10 +230,21 @@ static PyObject *PkgDepCacheSetCandidateVer(PyObject *Self,PyObject *Args)
 			&PyVersion_Type, &VersionObj) == 0)
       return 0;
 
+   pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+
+   VALIDATE_ITERATOR(Pkg);
+
    pkgCache::VerIterator &I = GetCpp<pkgCache::VerIterator>(VersionObj);
    if(I.end()) {
       return HandleErrors(PyBool_FromLong(false));
    }
+   VALIDATE_ITERATOR(I);
+
+   if (I.ParentPkg() != Pkg) {
+      PyErr_SetString(PyExc_ValueError, "Version does not belong to package");
+      return nullptr;
+   }
+
    depcache->SetCandidateVersion(I);
 
    return HandleErrors(PyBool_FromLong(true));
@@ -240,6 +259,9 @@ static PyObject *PkgDepCacheGetCandidateVer(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+
+   VALIDATE_ITERATOR(Pkg);
+
    pkgDepCache::StateCache & State = (*depcache)[Pkg];
    pkgCache::VerIterator I = State.CandidateVerIter(*depcache);
 
@@ -333,6 +355,7 @@ static PyObject *PkgDepCacheMarkKeep(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    depcache->MarkKeep(Pkg);
 
    Py_INCREF(Py_None);
@@ -349,6 +372,7 @@ static PyObject *PkgDepCacheSetReInstall(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    depcache->SetReInstall(Pkg,value);
 
    Py_INCREF(Py_None);
@@ -366,6 +390,7 @@ static PyObject *PkgDepCacheMarkDelete(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    depcache->MarkDelete(Pkg,purge);
 
    Py_INCREF(Py_None);
@@ -384,8 +409,10 @@ static PyObject *PkgDepCacheMarkInstall(PyObject *Self,PyObject *Args)
 			&autoInst, &fromUser) == 0)
       return 0;
 
-   Py_BEGIN_ALLOW_THREADS
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
+
+   Py_BEGIN_ALLOW_THREADS
    depcache->MarkInstall(Pkg, autoInst, 0, fromUser);
    Py_END_ALLOW_THREADS
 
@@ -403,6 +430,7 @@ static PyObject *PkgDepCacheMarkAuto(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    depcache->MarkAuto(Pkg,value);
 
    Py_INCREF(Py_None);
@@ -418,6 +446,7 @@ static PyObject *PkgDepCacheIsUpgradable(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Upgradable()));
@@ -432,6 +461,7 @@ static PyObject *PkgDepCacheIsGarbage(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Garbage));
@@ -446,6 +476,7 @@ static PyObject *PkgDepCacheIsAutoInstalled(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Flags & pkgCache::Flag::Auto));
@@ -460,6 +491,7 @@ static PyObject *PkgDepCacheIsNowBroken(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.NowBroken()));
@@ -474,6 +506,7 @@ static PyObject *PkgDepCacheIsInstBroken(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.InstBroken()));
@@ -489,6 +522,7 @@ static PyObject *PkgDepCacheMarkedInstall(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.NewInstall()));
@@ -504,6 +538,7 @@ static PyObject *PkgDepCacheMarkedUpgrade(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Upgrade()));
@@ -518,6 +553,7 @@ static PyObject *PkgDepCacheMarkedDelete(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Delete()));
@@ -532,6 +568,7 @@ static PyObject *PkgDepCacheMarkedKeep(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Keep()));
@@ -546,6 +583,7 @@ static PyObject *PkgDepCacheMarkedDowngrade(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    return HandleErrors(PyBool_FromLong(state.Downgrade()));
@@ -560,6 +598,7 @@ static PyObject *PkgDepCacheMarkedReinstall(PyObject *Self,PyObject *Args)
       return 0;
 
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    pkgDepCache::StateCache &state = (*depcache)[Pkg];
 
    bool res = state.Install() && (state.iFlags & pkgDepCache::ReInstall);
@@ -803,6 +842,8 @@ PyTypeObject PyDepCache_Type =
 
 									/*}}}*/
 
+#undef VALIDATE_ITERATOR
+#define VALIDATE_ITERATOR(I) (void) 0     // FIXME: Need access to depcache of pkgProblemResolver
 
 // pkgProblemResolver Class						/*{{{*/
 // ---------------------------------------------------------------------
@@ -863,6 +904,7 @@ static PyObject *PkgProblemResolverProtect(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O!",&PyPackage_Type,&PackageObj) == 0)
       return 0;
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    fixer->Protect(Pkg);
    Py_INCREF(Py_None);
    return HandleErrors(Py_None);
@@ -875,6 +917,7 @@ static PyObject *PkgProblemResolverRemove(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O!",&PyPackage_Type,&PackageObj) == 0)
       return 0;
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    fixer->Remove(Pkg);
    Py_INCREF(Py_None);
    return HandleErrors(Py_None);
@@ -887,6 +930,7 @@ static PyObject *PkgProblemResolverClear(PyObject *Self,PyObject *Args)
    if (PyArg_ParseTuple(Args,"O!",&PyPackage_Type,&PackageObj) == 0)
       return 0;
    pkgCache::PkgIterator &Pkg = GetCpp<pkgCache::PkgIterator>(PackageObj);
+   VALIDATE_ITERATOR(Pkg);
    fixer->Clear(Pkg);
    Py_INCREF(Py_None);
    return HandleErrors(Py_None);
