@@ -342,7 +342,10 @@ static PyObject *TagFileNext(PyObject *Self)
    //  append a \n because GetSection() will only give us a single \n
    //  but Scan() needs \n\n to work
    Obj.Section->Data = new char[Stop-Start+2];
-   snprintf(Obj.Section->Data, Stop-Start+2, "%s\n", Start);
+
+   memcpy(Obj.Section->Data, Start, Stop - Start);
+   Obj.Section->Data[Stop-Start] = '\n';
+   Obj.Section->Data[Stop-Start+1] = '\0';
    // Rescan it
    if(Obj.Section->Object.Scan(Obj.Section->Data, Stop-Start+2) == false)
       return HandleErrors(NULL);
@@ -491,6 +494,7 @@ static PyObject *TagSecNew(PyTypeObject *type,PyObject *Args,PyObject *kwds) {
 
 static PyObject *TagFileNew(PyTypeObject *type,PyObject *Args,PyObject *kwds)
 {
+   TagFileData *New;
    PyObject *File = 0;
    char Bytes = 0;
 
@@ -514,7 +518,7 @@ static PyObject *TagFileNew(PyTypeObject *type,PyObject *Args,PyObject *kwds)
       return 0;
    }
 
-   PyApt_UniqueObject<TagFileData> New((TagFileData*)type->tp_alloc(type, 0));
+   New = (TagFileData*)type->tp_alloc(type, 0);
    if (fileno != -1)
    {
 #ifdef APT_HAS_GZIP
@@ -552,7 +556,7 @@ static PyObject *TagFileNew(PyTypeObject *type,PyObject *Args,PyObject *kwds)
    // Create the section
    New->Section = (TagSecData*)(&PyTagSection_Type)->tp_alloc(&PyTagSection_Type, 0);
    new (&New->Section->Object) pkgTagSection();
-   New->Section->Owner = New.get();
+   New->Section->Owner = New;
    Py_INCREF(New->Section->Owner);
    New->Section->Data = 0;
    New->Section->Bytes = Bytes;
@@ -561,7 +565,7 @@ static PyObject *TagFileNew(PyTypeObject *type,PyObject *Args,PyObject *kwds)
    Py_XINCREF(New->Section->Encoding);
 #endif
 
-   return HandleErrors(New.release());
+   return HandleErrors(New);
 }
 									/*}}}*/
 // RewriteSection - Rewrite a section..					/*{{{*/
