@@ -11,7 +11,7 @@ for pkg in cache:
             for enhances in or_list.or_dependencies:
                 s += " %s" % enhances.name
                 if (enhances.name in cache and
-                    not cache[enhances.name].is_installed):
+                        not cache[enhances.name].is_installed):
                     s += "(*missing*) "
                 s += ","
-            print s[:-1]
+            print(s[:-1])
